<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 3/13/17
 * Time: 9:29 PM
 */

namespace qfq;

//use TYPO3\CMS\Core\FormProtection\Exception;

//require_once(__DIR__ . '/../store/Sip.php');
//require_once(__DIR__ . '/../store/Session.php');
require_once(__DIR__ . '/../store/Store.php');
require_once(__DIR__ . '/../Constants.php');
//require_once(__DIR__ . '/../helper/Ldap.php');
require_once(__DIR__ . '/../database/Database.php');
require_once(__DIR__ . '/../Evaluate.php');


/**
 * Class DragAndDrop
 * @package qfq
 */
class DragAndDrop {

    /**
     * @var Database instantiated class
     */
    private $db = null;

    /**
     * @var Store
     */
    private $store = null;

    /**
     * @var Evaluate instantiated class
     */
    protected $evaluate = null;  // copy of the loaded form

    /**
     * @var array
     */
    private $formSpec = null;

    /**
     * @param array $formSpec F_TABLE_NAME, F_DRAG_AND_DROP_ORDER_SQL, F_DRAG_AND_DROP_INTERVAL
     * @param bool|false $phpUnit
     *
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     */
    public function __construct(array $formSpec = array(), $phpUnit = false) {

        $this->formSpec = $formSpec;

        $dbIndex = DB_INDEX_DEFAULT;  //TODO hier muss noch die aktuelle DB ermittelt werden (kann im Form angegeben sein) - Gerade im Formular FORM Editor genau testen!
        $this->db = new Database($dbIndex);

        $this->store = Store::getInstance('', $phpUnit);
//        $this->evaluate = new Evaluate($this->store, $this->db);
    }

    /**
     * Reorder the elements according formSpec[F_DRAG_AND_DROP_ORDER_SQL]
     *
     * @return array|int
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     * @throws UserReportException
     */
    public function process() {

        $dragId = $this->store->getVar(DND_DRAG_ID, STORE_CLIENT, SANITIZE_ALLOW_ALNUMX);
        $setTo = $this->store->getVar(DND_SET_TO, STORE_CLIENT, SANITIZE_ALLOW_ALNUMX);
        $hoverId = $this->store->getVar(DND_HOVER_ID, STORE_CLIENT, SANITIZE_ALLOW_ALNUMX);

        $orderInterval = empty($this->formSpec[F_ORDER_INTERVAL]) ? 1 : $this->formSpec[F_ORDER_INTERVAL];
        $orderColumn = empty($this->formSpec[F_ORDER_COLUMN]) ? F_ORDER_COLUMN_NAME : $this->formSpec[F_ORDER_COLUMN];

        if (!is_array($this->formSpec[F_DRAG_AND_DROP_ORDER_SQL])) {
            return [];
        }

        $this->reorder($this->formSpec[F_DRAG_AND_DROP_ORDER_SQL], $dragId, $setTo, $hoverId, $orderColumn, $orderInterval, $this->formSpec[F_TABLE_NAME]);

        return [];
    }

    /**
     * @param array $rows
     * @param $dragId
     * @param $setTo
     * @param $hoverId
     * @param $orderColumn
     * @param $orderInterval
     * @param $tableName
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     */
    private function reorder(array $rows, $dragId, $setTo, $hoverId, $orderColumn, $orderInterval, $tableName) {
        $ord = $orderInterval;
        $ordDragOld = -1;

        // Reorder. Get index for 'drag' and 'hover'
        foreach ($rows as $key => $row) {

            // the dragged element: skip old position.
            if ($row[DND_COLUMN_ID] == $dragId) {
                $ordDragOld = $row[DND_COLUMN_ORD];
                continue;
            }

            // the dragged element: new position.
            if ($row[DND_COLUMN_ID] == $hoverId) {

                switch ($setTo) {
                    case DND_SET_TO_BEFORE:
                        $this->setNewOrder($tableName, $orderColumn, $dragId, $ordDragOld, $ord);
                        $ord += $orderInterval;
                        $this->setNewOrder($tableName, $orderColumn, $row[DND_COLUMN_ID], $row[DND_COLUMN_ORD], $ord);
                        break;

                    case DND_SET_TO_AFTER:
                        $this->setNewOrder($tableName, $orderColumn, $row[DND_COLUMN_ID], $row[DND_COLUMN_ORD], $ord);
                        $ord += $orderInterval;
                        $this->setNewOrder($tableName, $orderColumn, $dragId, $ordDragOld, $ord);
                        break;

                    default:
                        throw new CodeException('Unkown setTo string', $setTo, ERROR_UNKNOWN_TOKEN);
                }
            } else {
                $this->setNewOrder($tableName, $orderColumn, $row[DND_COLUMN_ID], $row[DND_COLUMN_ORD], $ord);
            }
            $ord += $orderInterval;
        }
    }

    /**
     * @param $tableName
     * @param $orderColumn
     * @param $id
     * @param $ordOld
     * @param $ordNew
     * @throws CodeException
     * @throws DbException
     * @throws UserFormException
     */
    private function setNewOrder($tableName, $orderColumn, $id, $ordOld, $ordNew) {

        if ($ordNew == $ordOld) {
            return;
        }

        $this->db->sql("UPDATE $tableName SET $orderColumn=? WHERE id=?", ROW_REGULAR, [$ordNew, $id]);
    }
}